@@ -1,10 +1,10 @@
-#Version:3.3.0
+#Version:3.3.1
 #Data: 2017/12/12
 #Author: Lilin Yin
 
 CMplot <- function(
 	Pmap,
-	col=c("dodgerblue1", "olivedrab3", "darkgoldenrod1", "red"),
+	col=c("#377EB8", "#4DAF4A", "#984EA3", "#FF7F00"),
 	bin.size=1e6,
 	bin.max=NULL,
 	pch=19,
@@ -34,10 +34,10 @@ CMplot <- function(
 	signal.line=1,
 	cir.chr=TRUE,
 	cir.chr.h=1.5,
-	cir.chr.col="black",
+	chr.den.col=c("darkgreen", "yellow", "red"),
 	cir.legend=TRUE,
 	cir.legend.cex=0.6,
-	cir.legend.col=c("darkgreen", "yellow", "red"),
+	cir.legend.col="black",
 	LOG10=TRUE,
 	box=FALSE,
 	conf.int.col="grey",
@@ -221,11 +221,6 @@ CMplot <- function(
 		if(!is.null(ylim)){
 			if(length(ylim)==1) ylim <- c(0,ylim)
 		}
-		if(length(cir.chr.col) != 1){
-			cir.density=TRUE
-		}else{
-			cir.density=FALSE
-		}
 		
 		if(is.null(conf.int.col))	conf.int.col <- NA
 		if(is.na(conf.int.col)){
@@ -344,6 +339,15 @@ CMplot <- function(
 
 		TotalN <- max(pvalue.posN)
 
+		if(length(chr.den.col) > 1){
+			cir.density=TRUE
+			den.fold <- 20
+			density.list <- Densitplot(map=Pmap[,c(1,1,2)], col=chr.den.col, plot=FALSE, bin=bin.size, legend.max=bin.max)
+			#list(den.col=col.seg, legend.col=legend.col, legend.y=legend.y)
+		}else{
+			cir.density=FALSE
+		}
+		
 		signal.line.index <- NULL
 		if(!is.null(threshold)){
 			if(!is.null(signal.line)){
@@ -425,10 +429,14 @@ CMplot <- function(
 							Y1chr=(RR)*cos(2*pi*(polygon.index)/TotalN)
 							X2chr=(RR+cir.chr.h)*sin(2*pi*(polygon.index)/TotalN)
 							Y2chr=(RR+cir.chr.h)*cos(2*pi*(polygon.index)/TotalN)
-							if(is.null(cir.chr.col)){
+							if(is.null(chr.den.col)){
 								polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=rep(colx,ceiling(length(chr)/length(colx)))[k],border=rep(colx,ceiling(length(chr)/length(colx)))[k])	
 							}else{
-								polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=cir.chr.col,border=cir.chr.col)
+								if(cir.density){
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col="grey",border="grey")
+								}else{
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=chr.den.col,border=chr.den.col)
+								}
 							}
 						}else{
 							polygon.index <- seq(1+round(band/2)+max(pvalue.posN.list[[k-1]]),-round(band/2)+max(pvalue.posN.list[[k]]), length=polygon.num)
@@ -436,17 +444,19 @@ CMplot <- function(
 							Y1chr=(RR)*cos(2*pi*(polygon.index)/TotalN)
 							X2chr=(RR+cir.chr.h)*sin(2*pi*(polygon.index)/TotalN)
 							Y2chr=(RR+cir.chr.h)*cos(2*pi*(polygon.index)/TotalN)
-							if(is.null(cir.chr.col)){
+							if(is.null(chr.den.col)){
 								polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=rep(colx,ceiling(length(chr)/length(colx)))[k],border=rep(colx,ceiling(length(chr)/length(colx)))[k])
 							}else{
-								polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=cir.chr.col,border=cir.chr.col)
+								if(cir.density){
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col="grey",border="grey")
+								}else{
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=chr.den.col,border=chr.den.col)
+								}
 							}		
 						}
 					}
 					
 					if(cir.density){
-						density.list <- Densitplot(map=Pmap[,c(1,1,2)], col=cir.chr.col, plot=FALSE, bin=bin.size, legend.max=bin.max)
-						#list(den.col=col.seg, legend.col=legend.col, legend.y=legend.y)
 
 						segments(
 							(RR)*sin(2*pi*(pvalue.posN-round(band/2))/TotalN),
@@ -464,6 +474,7 @@ CMplot <- function(
 							x.intersp=1,
 							yjust=0.5, xjust=0, xpd=TRUE
 						)
+						
 					}
 					
 					# XLine=(RR+cir.chr.h)*sin(2*pi*(1:TotalN)/TotalN)
@@ -489,10 +500,15 @@ CMplot <- function(
 					}
 					segments(0,r+H*(i-1)+cir.band*(i-1),0,r+H*i+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
 					segments(0,r+H*(i-1)+cir.band*(i-1),H/20,r+H*(i-1)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-1)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.75)+cir.band*(i-1),H/20,r+H*(i-0.75)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.75)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.5)+cir.band*(i-1),H/20,r+H*(i-0.5)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.5)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.25)+cir.band*(i-1),H/20,r+H*(i-0.25)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.25)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0)+cir.band*(i-1),H/20,r+H*(i-0)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					text(-r/15,r+H*(i-0.75)+cir.band*(i-1),round(Max*0.25,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
 					text(-r/15,r+H*(i-0.5)+cir.band*(i-1),round(Max*0.5,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
 					text(-r/15,r+H*(i-0.25)+cir.band*(i-1),round(Max*0.75,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
@@ -612,13 +628,13 @@ CMplot <- function(
 							Y1chr=(2*cir.band+RR)*cos(2*pi*(polygon.index)/TotalN)
 							X2chr=(2*cir.band+RR+cir.chr.h)*sin(2*pi*(polygon.index)/TotalN)
 							Y2chr=(2*cir.band+RR+cir.chr.h)*cos(2*pi*(polygon.index)/TotalN)
-								if(is.null(cir.chr.col)){
+								if(is.null(chr.den.col)){
 									polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=rep(colx,ceiling(length(chr)/length(colx)))[k],border=rep(colx,ceiling(length(chr)/length(colx)))[k])	
 								}else{
 									if(cir.density){
 										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col="grey",border="grey")
 									}else{
-										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=cir.chr.col,border=cir.chr.col)
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=chr.den.col,border=chr.den.col)
 									}
 								}
 						}else{
@@ -627,20 +643,18 @@ CMplot <- function(
 							Y1chr=(2*cir.band+RR)*cos(2*pi*(polygon.index)/TotalN)
 							X2chr=(2*cir.band+RR+cir.chr.h)*sin(2*pi*(polygon.index)/TotalN)
 							Y2chr=(2*cir.band+RR+cir.chr.h)*cos(2*pi*(polygon.index)/TotalN)
-							if(is.null(cir.chr.col)){
+							if(is.null(chr.den.col)){
 								polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=rep(colx,ceiling(length(chr)/length(colx)))[k],border=rep(colx,ceiling(length(chr)/length(colx)))[k])	
 							}else{
 									if(cir.density){
 										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col="grey",border="grey")
 									}else{
-										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=cir.chr.col,border=cir.chr.col)
+										polygon(c(rev(X1chr),X2chr),c(rev(Y1chr),Y2chr),col=chr.den.col,border=chr.den.col)
 									}
 							}	
 						}
 					}
 					if(cir.density){
-						density.list <- Densitplot(map=Pmap[,c(1,1,2)], col=cir.chr.col, plot=FALSE, bin=bin.size, legend.max=bin.max)
-						#list(den.col=col.seg, legend.col=legend.col, legend.y=legend.y)
 
 						segments(
 							(2*cir.band+RR)*sin(2*pi*(pvalue.posN-round(band/2))/TotalN),
@@ -658,6 +672,7 @@ CMplot <- function(
 							x.intersp=1,
 							yjust=0.5, xjust=0, xpd=TRUE
 						)
+						
 					}
 					
 					if(cir.density){
@@ -680,10 +695,15 @@ CMplot <- function(
 					}
 					segments(0,r+H*(i-1)+cir.band*(i-1),0,r+H*i+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
 					segments(0,r+H*(i-1)+cir.band*(i-1),H/20,r+H*(i-1)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-1)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.75)+cir.band*(i-1),H/20,r+H*(i-0.75)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.75)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.5)+cir.band*(i-1),H/20,r+H*(i-0.5)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.5)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0.25)+cir.band*(i-1),H/20,r+H*(i-0.25)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0.25)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					segments(0,r+H*(i-0)+cir.band*(i-1),H/20,r+H*(i-0)+cir.band*(i-1),col=cir.legend.col,lwd=1.5)
+					circle.plot(myr=r+H*(i-0)+cir.band*(i-1),lwd=0.5,add=TRUE,col='grey')
 					text(-r/15,r+H*(i-0.25)+cir.band*(i-1),round(Max*0.25,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
 					text(-r/15,r+H*(i-0.5)+cir.band*(i-1),round(Max*0.5,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
 					text(-r/15,r+H*(i-0.75)+cir.band*(i-1),round(Max*0.75,round.n),adj=1,col=cir.legend.col,cex=cir.legend.cex,font=2)
@@ -848,16 +868,33 @@ CMplot <- function(
 							}
 						}
 						if(Max<=1){
-							plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+							if(cir.density){
+								plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold, Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+									cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+							}else{
+								plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
 								cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+							}
 						}else{
-							plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+1),ylab=ylab,
+							if(cir.density){
+								plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold,Max+1),ylab=ylab,
 								cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+							}else{
+								plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)),ylim=c(0,Max+1),ylab=ylab,
+								cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+							}
 						}
 					}else{
-						plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=ylim,ylab=ylab,
+						Max <- max(ylim)
+						if(cir.density){
+							plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold, max(ylim)),ylab=ylab,
 							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+						}else{
+							plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)),ylim=ylim,ylab=ylab,
+							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main=paste("Manhattan plot of",taxa[i]))
+						}
 					}
+					
 					if(is.null(chr.labels)){
 						axis(1, at=c(0,ticks),cex.axis=cex.axis,font=2,labels=c("Chr",chr.ori))
 					}else{
@@ -867,14 +904,18 @@ CMplot <- function(
 						if(Max>1){
 							#print(seq(0,(Max+1),ceiling((Max+1)/10)))
 							axis(2,at=seq(0,(Max+1),ceiling((Max+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(Max+1),ceiling((Max+1)/10)))
+							legend.y <- tail(seq(0,(Max+1),ceiling((Max+1)/10)), 1)
 						}else{
 							axis(2,at=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))),cex.axis=cex.axis,font=2,labels=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))))
+							legend.y <- tail(seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))), 1)
 						}
 					}else{
 						if(ylim[2]>1){
-							axis(2,at=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)))
+							axis(2,at=seq(0,ylim[2],ceiling((ylim[2])/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]),ceiling((ylim[2])/10)))
+							legend.y <- tail(ylim[2], 1)
 						}else{
-							axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
+							axis(2,at=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))))
+							legend.y <- tail(ylim[2], 1)
 						}
 					}
 					if(!is.null(threshold)){
@@ -936,6 +977,32 @@ CMplot <- function(
 							}
 						}
 					}
+					if(cir.density){
+						for(yll in 1:length(pvalue.posN.list)){
+							polygon(c(min(pvalue.posN.list[[yll]]), min(pvalue.posN.list[[yll]]), max(pvalue.posN.list[[yll]]), max(pvalue.posN.list[[yll]])), 
+								c(-0.5*Max/den.fold, -1.5*Max/den.fold, 
+								-1.5*Max/den.fold, -0.5*Max/den.fold), 
+								col="grey", border="grey")
+						}
+						
+						segments(
+							pvalue.posN,
+							-0.5*Max/den.fold,
+							pvalue.posN,
+							-1.5*Max/den.fold,
+							col=density.list$den.col, lwd=0.1
+						)
+						legend(
+							x=max(pvalue.posN),
+							y=legend.y,
+							title="", legend=density.list$legend.y, pch=15, pt.cex = 2.5, col=density.list$legend.col,
+							cex=0.8, bty="n",
+							y.intersp=1,
+							x.intersp=1,
+							yjust=1, xjust=0, xpd=TRUE
+						)
+						
+					}
 				if(box) box()
 				#if(!is.null(threshold) & (length(grep("FarmCPU",taxa[i])) != 0))	abline(v=which(pvalueT[,i] < min(threshold)/max(dim(Pmap))),col="grey",lty=2,lwd=signal.line)
 				if(file.output)  dev.off()
@@ -995,14 +1062,16 @@ CMplot <- function(
 					}
 					xn <- ifelse(R == 1, R, R * 2/3)
 					if(Max<=1){
-						plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+						plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
 							cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 					}else{
-						plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+1),ylab=ylab,
+						plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+band),ylim=c(0,Max+1),ylab=ylab,
 							cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 					}
 				}else{
-					plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+2*band),ylim=ylim,ylab=ylab,
+					xn <- ifelse(R == 1, R, R * 2/3)
+					Max <- max(ylim)
+					plot(pvalue.posN,logpvalue,pch=pch,cex=cex[2]*xn,col=rep(rep(colx,N[i]),add[[i]]),xlim=c(0,max(pvalue.posN)+band),ylim=ylim,ylab=ylab,
 						cex.axis=cex.axis*xn,cex.lab=2*xn,font=2,axes=FALSE)
 				}
 				
@@ -1028,9 +1097,9 @@ CMplot <- function(
 					}
 				}else{
 					if(ylim[2]>1){
-						axis(2,at=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)),cex.axis=cex.axis*xn,font=2,labels=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)))
+						axis(2,at=seq(0,(ylim[2]),ceiling((ylim[2])/10)),cex.axis=cex.axis*xn,font=2,labels=seq(0,(ylim[2]),ceiling((ylim[2])/10)))
 					}else{
-						axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis*xn,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
+						axis(2,at=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis*xn,font=2,labels=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))))
 					}
 				}
 				if(!is.null(threshold)){
@@ -1101,7 +1170,7 @@ CMplot <- function(
 				par(mar = c(5,6,4,3),xaxs=xaxs,yaxs=yaxs,xpd=TRUE)
 			}
 			if(!file.output){
-				if(is.null(dev.list()))	dev.new(width = 15, height = 6)
+				dev.new(width = 15, height = 6)
 				par(xpd=TRUE)
 			}
 			
@@ -1142,19 +1211,33 @@ CMplot <- function(
 					}
 				}
 				if(Max<=1){
-					plot(NULL,xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
-						cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot")
+					if(cir.density){
+						plot(NULL,xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold, Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot")
+					}else{
+						plot(NULL,xlim=c(0,max(pvalue.posN)),ylim=c(0,Max+10^(-ceiling(-log10(Max)))),ylab=ylab,
+							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot")
+					}
 				}else{
-					plot(NULL,xlim=c(0,max(pvalue.posN)+2*band),ylim=c(0,Max+1),ylab=ylab,
-						cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+					if(cir.density){
+						plot(NULL,xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold,Max+1),ylab=ylab,
+							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+					}else{
+						plot(NULL,xlim=c(0,max(pvalue.posN)),ylim=c(0,Max+1),ylab=ylab,
+							cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+					}
 				}
 			}else{
-				plot(NULL,xlim=c(0,max(pvalue.posN)+2*band),ylim=ylim,ylab=ylab,
-					cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+				Max <- max(ylim)
+				if(cir.density){
+					plot(NULL,xlim=c(0,1.01*max(pvalue.posN)),ylim=c(-Max/den.fold,Max+1),ylab=ylab,
+						cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+				}else{
+					plot(NULL,xlim=c(0,max(pvalue.posN)),ylim=ylim,ylab=ylab,
+						cex.axis=cex.axis,cex.lab=2,font=2,axes=FALSE,xlab=xlab,main="Manhattan plot of")
+				}
 			}
-			
 			legend("topleft",taxa,col=t(col)[1:R],pch=19,text.font=6,box.col=NA)
-
 			if(is.null(chr.labels)){
 				axis(1, at=c(0,ticks),cex.axis=cex.axis,font=2,labels=c("Chr",chr.ori))
 			}else{
@@ -1164,14 +1247,18 @@ CMplot <- function(
 				if(Max>1){
 					#print(seq(0,(Max+1),ceiling((Max+1)/10)))
 					axis(2,at=seq(0,(Max+1),ceiling((Max+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(Max+1),ceiling((Max+1)/10)))
+					legend.y <- tail(seq(0,(Max+1),ceiling((Max+1)/10)), 1)
 				}else{
 					axis(2,at=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))),cex.axis=cex.axis,font=2,labels=seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))))
+					legend.y <- tail(seq(0,Max+10^(-ceiling(-log10(Max))),10^(-ceiling(-log10(Max)))), 1)
 				}
 			}else{
 				if(ylim[2]>1){
-					axis(2,at=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]+1),ceiling((ylim[2]+1)/10)))
+					axis(2,at=seq(0,(ylim[2]),ceiling((ylim[2])/10)),cex.axis=cex.axis,font=2,labels=seq(0,(ylim[2]),ceiling((ylim[2])/10)))
+					legend.y <- tail(ylim[2], 1)
 				}else{
-					axis(2,at=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2]+10^(-ceiling(-log10(ylim[2]))),10^(-ceiling(-log10(ylim[2])))))
+					axis(2,at=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))),cex.axis=cex.axis,font=2,labels=seq(0,ylim[2],10^(-ceiling(-log10(ylim[2])))))
+					legend.y <- tail(ylim[2], 1)
 				}
 			}
 			do <- TRUE
@@ -1190,15 +1277,17 @@ CMplot <- function(
 					}
 					sam.index[[i]] <- sam.index[[i]][-which(sam.index[[i]] %in% plot.index)]
 					logpvalue=logpvalueT[plot.index,i]
-					points(pvalue.posN[plot.index],logpvalue,pch=pch,cex=cex[2],col=t(col)[i])
+					points(pvalue.posN[plot.index],logpvalue,pch=pch,cex=cex[2],col=rgb(col2rgb(t(col)[i])[1], col2rgb(t(col)[i])[2], col2rgb(t(col)[i])[3], 100, maxColorValue=255))
 					#if(!is.null(threshold) & (length(grep("FarmCPU",taxa[i])) != 0))	abline(v=which(pvalueT[,i] < min(threshold)/max(dim(Pmap))),col="grey",lty=2,lwd=signal.line)
 				}
 				if(length(sam.index[[i]]) == 0) do <- FALSE
 			}
+			
 			# for(i in 1:R){
 				# logpvalue=logpvalueT[,i]
 				# points(pvalue.posN,logpvalue,pch=pch,cex=cex[2],col=t(col)[i])
 			# }
+			
 			if(!is.null(threshold)){
 				if(sum(threshold!=0)==length(threshold)){
 					for(thr in 1:length(threshold)){
@@ -1207,7 +1296,32 @@ CMplot <- function(
 					}
 				}
 			}
-
+			if(cir.density){
+						for(yll in 1:length(pvalue.posN.list)){
+							polygon(c(min(pvalue.posN.list[[yll]]), min(pvalue.posN.list[[yll]]), max(pvalue.posN.list[[yll]]), max(pvalue.posN.list[[yll]])), 
+								c(-0.5*Max/den.fold, -1.5*Max/den.fold, 
+								-1.5*Max/den.fold, -0.5*Max/den.fold), 
+								col="grey", border="grey")
+						}
+						
+						segments(
+							pvalue.posN,
+							-0.5*Max/den.fold,
+							pvalue.posN,
+							-1.5*Max/den.fold,
+							col=density.list$den.col, lwd=0.1
+						)
+						legend(
+							x=max(pvalue.posN),
+							y=legend.y,
+							title="", legend=density.list$legend.y, pch=15, pt.cex = 2.5, col=density.list$legend.col,
+							cex=0.8, bty="n",
+							y.intersp=1,
+							x.intersp=1,
+							yjust=1, xjust=0, xpd=TRUE
+						)
+						
+			}
 			if(file.output) dev.off()
 			
 		}
@@ -1302,7 +1416,7 @@ CMplot <- function(
 					if(file=="tiff")	tiff(paste("Multraits.QQplot.",paste(taxa,collapse="."),".tiff",sep=""), width = 5.5*dpi,height=5.5*dpi,res=dpi)
 					par(mar = c(5,5,4,2),xpd=TRUE)
 				}else{
-					if(is.null(dev.list()))	dev.new(width = 5.5, height = 5.5)
+					dev.new(width = 5.5, height = 5.5)
 					par(xpd=TRUE)
 				}
 				p_value_quantiles=(1:nrow(Pmap))/(nrow(Pmap)+1)
@@ -1460,4 +1574,5 @@ CMplot <- function(
 			}
 		}
 	}
+	if(file.output)	print(paste("Plots are stored in: ", getwd(), sep=""))
 }
